#!/usr/bin/env python

"""Look up IPv6 addresses for hostnames."""

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import socket
import sys

from dpkt.ip6 import IP6Address

def lookup(host):
    """Returns list of addresses for given hostname."""
    _, _, _, _, sockaddrs = zip(*socket.getaddrinfo(host, None, socket.AF_INET6, socket.SOCK_STREAM, socket.SOL_TCP))
    addrs, _, _, _ = zip(*sockaddrs)
    return [IP6Address(ascii=addr) for addr in addrs]

def lookup_or_die(host, code=1):
    """Returns list of addresses for given hostname.

    If the hostname cannot be resolved, an error is printed to stderr and exit
    is called with the given exit code.
    """
    try:
        return lookup(host)
    except socket.gaierror as e:
        sys.stderr.write('%s: %s\n' % (host, e.strerror))
        exit(code)

def usage(code=1):
    sys.stderr.write('Usage: %s HOST\n' % sys.argv[0])
    exit(code)

def main():
    if len(sys.argv) != 2:
        usage()
    host = sys.argv[1]
    addrs = lookup_or_die(host)
    for addr in addrs:
        print addr.ascii

if __name__ == '__main__':
    main()
