#!/usr/bin/env python

"""Send ICMPv6 'message too big' messages."""

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import socket
import sys

from dpkt.ip6 import IP6
from dpkt.icmp6 import ICMP6, ICMP6_PACKET_TOO_BIG

from icmp6 import receive_icmps
from nslookup6 import lookup_or_die
from sequence import ICMP6Sequence
from sniffing import start_sniffing

def too_big(p, mtu, src, dst):
    """Return an ICMP 'message too big' packet from 'src' to 'dst' in response
    to packet 'p' sending mtu 'mtu'.
    """
    load = str(p)[:1232]
    return IP6(plen=len(load) + 8, nxt=socket.IPPROTO_ICMPV6, src=src, dst=dst,
               p=socket.IPPROTO_ICMPV6, data=
               ICMP6(type=ICMP6_PACKET_TOO_BIG, data=
               ICMP6.TooBig(mtu=mtu, data=load)))

def ping(sniffer, addr, mtu):
    seq = ICMP6Sequence(addr, iface=sniffer.iface)
    timeout = sniffer.timeout()
    while True:
        timeout.add_time(1.0)
        request = seq.next_packet_testing_mtu(mtu)
        print 'Sending %d...' % request.extension_hdrs[44].id
        sniffer.send(request, True)
        p = None
        for p in receive_icmps(sniffer, request, timeout, True):
            frag = p.extension_hdrs[44]
            if frag is not None:
                print '%d+%d' % (frag.id, frag.frag_off_resv_m & 0xfff8)
            else:
                print '*+0'
                sniffer.send(too_big(p, max(mtu - 1, 1260), request.src, request.dst))
        if p is None:
            print '*'
        sys.stdout.flush()

def usage(code=1):
    sys.stderr.write('Usage: %s HOST MTU\n' % sys.argv[0])
    exit(code)

def main():
    try:
        if len(sys.argv) != 3:
            usage()
        host = sys.argv[1]
        try:
            mtu = int(sys.argv[2])
        except ValueError:
            usage()
        addr = lookup_or_die(host)[0]
        start_sniffing(ping, True, args=(addr, mtu))
    except KeyboardInterrupt:
        sys.stderr.write('Received SIGINT\n')

if __name__ == '__main__':
    main()
